import django
try:
    from django.db.backends import utils as util
except ImportError:
    from django.db.backends import util

from django_xray.patches.utils import patch_method
from django_xray.traces import trace_sql


def key(db, attr):
    return 'db.%s.%s.%s' % (db.client.executable_name, db.alias, attr)


def _get_query_type(query):
    return (query.split(None, 1) or ['__empty__'])[0].lower()


def patched_execute(orig_execute, self, query, *args, **kwargs):
    name = key(self.db, 'execute.%s' % _get_query_type(query))
    with trace_sql(name, self.db, query):
        return orig_execute(self, query, *args, **kwargs)


def patched_executemany(orig_executemany, self, query, *args, **kwargs):
    name = key(self.db, 'executemany.%s' % _get_query_type(query))
    with trace_sql(name, self.db, query):
        return orig_executemany(self, query, *args, **kwargs)


def patched_callproc(orig_callproc, self, query, *args, **kwargs):
    name = key(self.db, 'callproc.%s' % _get_query_type(query))
    with trace_sql(name, self.db, query):
        return orig_callproc(self, query, *args, **kwargs)


def patch():
    """
    The CursorWrapper is a pretty small wrapper around the cursor.  If
    you are NOT in debug mode, this is the wrapper that's used.  Sadly
    if it's in debug mode, we get a different wrapper for version
    earlier than 1.6.
    """

    patch_method(util.CursorWrapper, 'execute')(patched_execute)
    patch_method(util.CursorWrapper, 'executemany')(patched_executemany)
    patch_method(util.CursorWrapper, 'callproc')(patched_callproc)



from setuptools import find_packages, setup

docs_require = [
    'sphinx>=1.4.0',
]

tests_require = [
    'coverage==.4.2',
    'attrs>=17.2.0',
    'pytest==3.0.5',
    'pytest-django==3.1.2',

    # Linting
    'isort==4.2.5',
    'flake8==3.0.3',
    'flake8-blind-except==0.1.1',
    'flake8-debugger==1.4.0',
]

setup(
    name='django-aws-xray',
    version='0.0.1',
    description="Something I made",
    long_description=open('README.rst', 'r').read(),
    url='https://github.com/mvantellingen/django-xray',
    author="Michael van Tellingen",
    author_email="",
    install_requires=[
        'Django>=1.8',
        'six>=1.1',
    ],
    tests_require=tests_require,
    extras_require={
        'docs': docs_require,
        'test': tests_require,
    },
    use_scm_version=True,
    entry_points={},
    package_dir={'': 'src'},
    packages=find_packages('src'),
    include_package_data=True,
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    zip_safe=False,
)
